
%
% loadOMM.m
%
% Defines a function loadOMM() that returns a description 
% of the Open Michaelis-Menten model. This model is the 
% first of three described in the Results section and is 
% used to demonstrate the flexibility of the py-
% substitution method.
%
% The structure of the OMM model is as follows:
%
%  Index     Species 
% =======   =====================================
%  x1        Enzyme
%  x2        Substrate
%  x3        Complex
%  x4        Product
%
%
%  Index     Reaction
% =======   =====================================
%  v1        E + S  --(v1)->  C
%  v2        C  --(v2)->  E + S
%  v3        C  --(v3)->  E + P
%  v4        null  --(v4)->  S
%  v5        P  --(v5)->  null
%
% loadOMM() takes no arguments, but returns the following
% values:
%
% S    - the 4 x 5 stoichiometric matrix that defines the 
%        OMM model reaction structure
% v    - the vector of mass action reaction velocities
% k    - the vector of reaction rate constants
% x    - the vector of species abundances.
% dxdt - the vector of mass balance equations, equal to 
%        the product S * v

function [S,v,k,x,dxdt] = loadOMM()

  % Species and reaction rate constants
  x = sym( 'x', [4,1] ); k = sym( 'k', [5,1] );
  k = sym( k, 'positive' ); k = sym( k, 'real' );
  x = sym( x, 'positive' ); x = sym( x, 'real' );

  % Reaction velocities
  v = [ ...
    k(1) * x(1) * x(2);
    k(2) * x(3);
    k(3) * x(3);
    k(4);
    k(5) * x(4);
  ];
  
  % Stoichiometric matrix
  S = [ ...
  
   -1  1  1  0  0;
   -1  1  0  1  0;
    1 -1 -1  0  0;
    0  0  1  0 -1;
  
  ];
  
  % Mass balance equations
  dxdt = S * v;

