
#
# loadXearm.mpl
#
# Returns the stoichiometric matrix S, reaction velocity vector vv,
# reaction rate constant vector vk, and species concentration
# vector vx for the extended extrinsic apoptosis reaction model,
# or xEARM.
# 

#  Index     Symbol      Meaning 
# =======   ========    =============================================
#  x( 1)     L           Ligand
#  x( 2)     R           Receptor
#  x( 3)     L:R         Ligand receptor complex
#  x( 4)     DISC        Activated receptor/DISC
#  x( 5)     flip        FLIP protein
#  x( 6)     flip:DISC   FLIP, activated receptor complex
#  x( 7)     pC8         Procaspase 8
#  x( 8)     DISC:pC8    Procaspase 8, actiavated receptor complex
#  x( 9)     C8          Active caspase 8
#  x(10)     Bar         Bar
#  x(11)     Bar:C8      Bar caspase 8 complex
#  x(12)     pC3         Procaspase 3
#  x(13)     C8:pC3      Caspase 8, procasepase 3 complex
#  x(14)     C3          Active caspase 3
#  x(15)     pC6         Procaspase 6
#  x(16)     C3:pC6      Caspase 3, procaspase 6 complex
#  x(17)     C6          Active caspase 6
#  x(18)     C6:pC8      Caspase 6, procaspase 8 complex
#  x(19)     XIAP        X-linked inhibitor of apoptosis
#  x(20)     XIAP:C3     XIAP, caspase 3 protein
#  x(21)     PARP        PARP
#  x(22)     C3:PARP     Caspase 3, PARP complex
#  x(23)     CPARP       Cleaved PARP protein
#  x(24)     Bid         Bid
#  x(25)     C8:Bid      Caspase 8, Bid complex
#  x(26)     tBid        Truncated Bid protein
#  x(27)     Bcl2c       Cytoplasmic Bcl2
#  x(28)     Bcl2c:tBid  Cytoplasmic Bcl2, tBid complex
#  x(29)     Bax         Cytoplasmic Bax
#  x(30)     tBid:Bax    tBid, Bax complex
#  x(31)     Bax*        Activated Bax
#  x(32)     Baxm        Mitochondrial Bax protein
#  x(33)     Bcl2        Mitochondrial Bcl2 protein
#  x(34)     Baxm:Bcl2   Bax, Bcl2 complex
#  x(35)     Bax2        Bax homodimer
#  x(36)     Bax2:Bcl2   Bax homodimer, Bcl2 complex
#  x(37)     Bax4        Bax homotetramer 
#  x(38)     Bax4:Bcl2   Bax homotetramer, Bcl2 complex
#  x(39)     Mito        Mitochondrial pore (closed)
#  x(40)     Bax4:M      Bax heterotetramer-bound pore
#  x(41)     AMito       Mitochondrial pore (open)
#  x(42)     mCytoC      Mitochondrial Cytochrome C
#  x(43)     AMito:mCytoC Cytochrome C bound to mitochondrial pore
#  x(44)     ACytoC      
#  x(45)     mSmac       Mitochondrial cSmac protein
#  x(46)     AMito:mSmac cSmac bound to mitochondrial pore
#  x(47)     ASmac       
#  x(48)     CytoC       Cytoplasmic cytochrome C
#  x(49)     Apaf        Apaf
#  x(50)     Apaf:CytoC  Apaf, Cytochrome C complex
#  x(51)     Apaf*       Activated Apaf
#  x(52)     pC9         Procaspase 9
#  x(53)     Apop        Apoptosome
#  x(54)     Apop:pC3    Apoptosome-bound procaspase 3
#  x(55)     cSmac        Cytoplasmic cSmac
#  x(56)     Apop:XIAP   Apoptosome-bound XIAP
#  x(57)     cSmac:XIAP   cSmac-bound XIAP
#  x(58)     C3_Ub       Poly-K48 ubiquitinated caspase 3 
#
#
# Chemical Reaction Fluxes
#
#  Index    Reaction
# =======   =====================================
#  v( 1)    L + R  --(v1)->  L:R
#  v( 2)    L:R    --(v2)->  L + R
#  v( 3)    L:R    --(v3)->  DISC
#
#  v( 4)    flip + DISC  --(v4)->  flip:DISC  
#  v( 5)    flip:DISC    --(v5)->  flip + DISC
#
#  v( 6)    pC8 + DISC  --(v6)->  DISC:pC8
#  v( 7)    DISC:pC8    --(v7)->  pC8 + DISC
#  v( 8)    DISC:pC8    --(v8)->  C8 + DISC
#
#  v( 9)    C8 + BAR  --(v9)->  BAR:C8 
#  v(10)    BAR:C8   --(v10)->  C8 + BAR
#
#  v(11)    pC3 + C8  --(v11)->  pC3:C8
#  v(12)    pC3:C8    --(v12)->  pC3 + C8
#  v(13)    pC3:C8    --(v13)->  C3 + C8
#
#  v(14)    pC6 + C3  --(v14)->  pC6:C3
#  v(15)    pC6:C3    --(v15)->  pC6 + C3
#  v(16)    pC6:C3    --(v16)->  C6 + C3
#
#  v(17)    pC8 + C6  --(v17)->  pC8:C6
#  v(18)    pC8:C6    --(v18)->  pC8 + C6
#  v(19)    pC8:C6    --(v19)->  C8 + C6
#
#  v(20)    XIAP + C3  --(v20)->  XIAP:C3
#  v(21)    XIAP:C3    --(v21)->  XIAP + C3
#  v(22)    XIAP:C3    --(v22)->  XIAP + C3_U
#
#  v(23)    PARP + C3  --(v23)->  PARP:C3
#  v(24)    PARP:C3    --(v24)->  PARP + C3
#  v(25)    PARP:C3    --(v25)->  CPARP + C3
#
#  v(26)    Bid + C8  --(v26)->  Bid:C8
#  v(27)    Bid:C8    --(v27)->  Bid + C8
#  v(28)    Bid:C8    --(v28)->  tBid + C8
#
#  v(29)    tBid + Bcl2c  --(v29)->  tBid:Bcl2c  
#  v(30)    tBid:Bcl2c    --(v30)->  tBid + Bcl2c  
#
#  v(31)    Bax + tBid  --(v31)->  Bax:tBid
#  v(32)    Bax:tBid    --(v32)->  Bax + tBid
#  v(33)    Bax:tBid    --(v33)->  aBax + tBid 
#
#  v(34)    aBax  --(v34)->  MBax 
#  v(35)    MBax  --(v35)->  aBax 
#
#  v(36)    MBax + Bcl2  --(v36)->  MBax:Bcl2  
#  v(37)    MBax:Bcl2    --(v37)->  MBax + Bcl2  
#
#  v(38)    MBax + MBax  --(v38)->  Bax2
#  v(39)    Bax2         --(v39)->  MBax + MBax
#
#  v(40)    Bax2 + Bcl2  --(v40)->  MBax2:Bcl2  
#  v(41)    MBax2:Bcl2   --(v41)->  Bax2 + Bcl2  
#
#  v(42)    Bax2 + Bax2  --(v42)->  Bax4
#  v(43)    Bax4         --(v43)->  Bax2 + Bax2
#
#  v(44)    Bax4 + Bcl2  --(v44)->  Bax4:Bcl2  
#  v(45)    Bax4:Bcl2    --(v45)->  Bax4 + Bcl2  
#
#  v(46)    Bax4 + Mit0  --(v46)->  Bax4:Mito
#  v(47)    Bax4:Mit0    --(v47)->  Bax4 + Mito
#  v(48)    Bax4:Mit0    --(v48)->  AMito  
#
#  v(49)    AMito + mCytoC  --(v49)->  AMito:mCytoC
#  v(50)    AMito:mCytoC    --(v50)->  AMito + mCytoC
#  v(51)    AMito:mCytoC    --(v51)->  AMito + ACytoC
#
#  v(52)    AMito + mSMac  --(v52)->  AMito:mSmac
#  v(53)    AMito:mSMac    --(v53)->  AMito + mSmac
#  v(54)    AMito + mSMac  --(v54)->  AMito + ASmac  
#
#  v(55)    ACytoC  --(v55)->  cCytoC
#  v(56)    cCytoC  --(v56)->  ACytoC
#
#  v(57)    Apaf + cCytoC  --(v57)->  Apaf:cCytoC  
#  v(58)    Apaf:cCytoC    --(v58)->  Apaf + cCytoC  
#  v(59)    Apaf:cCytoC    --(v59)->  Apaf* + cCytoC  
#
#  v(60)    Apaf* + Procasp9  --(v60)->  Apoptosome  
#  v(61)    Apoptosome        --(v61)->  Apaf* + Procasp9
#
#  v(62)    Apop + pC3  --(v62)->  Apop:pC3
#  v(63)    Apop:pC3    --(v63)->  Apop + pC3
#  v(64)    Apop:pC3    --(v64)->  Apop + C3  
#
#  v(65)    ASmac  --(v65)->  cSmac
#  v(66)    cSmac  --(v66)->  ASmac
#
#  v(67)    Apop + XIAP  --(v67)->  Apop:XIAP  
#  v(68)    Apop:XIAP    --(v68)->  Apop + XIAP  
#
#  v(69)    cSmac + XIAP  --(v69)->  cSmac:XIAP  
#  v(70)    cSmac:XIAP    --(v70)->  cSmac + XIAP  
#
#
# Synthesis and Degradation Fluxes
#
#  Index    Reaction
# =======   =====================================
#  v(71)    null       --(v71)->  R
#  v(72)    R          --(v72)->  null
#  v(73)    null       --(v73)->  flip
#  v(74)    flip       --(v74)->  null
#  v(75)    flip:DISC  --(v75)->  null
#
#  v(76)    null    --(v76)->  pC8
#  v(77)    pC8     --(v77)->  null
#  v(78)    null    --(v78)->  BAR
#  v(79)    BAR     --(v79)->  null
#  v(80)    BAR:C8  --(v80)->  null
#
#  v(81)    null        --(v81)->  Bid
#  v(82)    Bid         --(v82)->  null
#  v(83)    null        --(v83)->  Bcl2c
#  v(84)    Bcl2c       --(v84)->  null
#  v(85)    tBid:Bcl2c  --(v85)->  null
#
#  v(86)    null       --(v86)->  Bax
#  v(87)    Bax        --(v87)->  null
#  v(88)    null       --(v88)->  Bcl2
#  v(89)    Bcl2       --(v89)->  null
#  v(90)    Baxm:Bcl2  --(v90)->  null
#  v(91)    Bax2:Bcl2  --(v91)->  null
#  v(92)    Bax4:Bcl2  --(v92)->  null
#
#  v(93)    AMito      --(v93)->  Mito
#  v(94)    Apaf*      --(v94)->  Apaf
#
#  v(95)    null        --(v95)->  XIAP
#  v(96)    XIAP        --(v96)->  null
#  v(97)    null        --(v97)->  mSmac
#  v(98)    mSmac       --(v98)->  null
#  v(99)    cSmac       --(v99)->  null
#  v(100)   cSmac:XIAP  --(v100)->  null
#
#  v(101)   null    --(v101)->  pC3
#  v(102)   pC3     --(v102)->  null
#  v(103)   C3_U    --(v103)->  null
#  v(104)   null    --(v104)->  pC6
#  v(105)   pC6     --(v105)->  null
#  v(106)   C6      --(v106)->  null
#
#  v(107)   Apop:XIAP  --(v107)->  Apop
#
#  v(108)   null   --(v108)->  Parp
#  v(109)   Parp   --(v109)->  null
#  v(110)   cParp  --(v110)->  null
#
#  v(111)   null  --(v111)->  L
#  v(112)   L     --(v112)->  null
#
#  v(113)   null    --(v113)->  mCytoC
#  v(114)   mCytoC  --(v114)->  null
#  v(115)   cCytoC  --(v115)->  null


loadXearm := proc()

  local S, vv, vk, vx, vc;
  local nx, nk, nc, ns;
  local kx;
  
  with( LinearAlgebra ):
  with( ArrayTools ):
  
  #
  # The stoichiometric matrix
  #
  
  S := Matrix( 58, 115 ):
  
  S[1,1]   := -1:
  S[1,2]   :=  1:
  S[1,111] :=  1:
  S[1,112] := -1:
  S[2,1]   := -1:
  S[2,2]   :=  1:
  S[2,71]  :=  1:
  S[2,72]  := -1:
  S[3,1]   :=  1:
  S[3,2]   := -1:
  S[3,3]   := -1:
  S[4,3]   :=  1:
  S[4,4]   := -1:
  S[4,5]   :=  1:
  S[4,6]   := -1:
  S[4,7]   :=  1:
  S[4,8]   :=  1:
  S[5,4]   := -1:
  S[5,5]   :=  1:
  S[5,73]  :=  1:
  S[5,74]  := -1:
  S[6,4]   :=  1:
  S[6,5]   := -1:
  S[6,75]  := -1:
  S[7,6]   := -1:
  S[7,7]   :=  1:
  S[7,17]  := -1:
  S[7,18]  :=  1:
  S[7,76]  :=  1:
  S[7,77]  := -1:
  S[8,6]   :=  1:
  S[8,7]   := -1:
  S[8,8]   := -1:
  S[9,8]   :=  1:
  S[9,9]   := -1:
  S[9,10]  :=  1:
  S[9,11]  := -1:
  S[9,12]  :=  1:
  S[9,13]  :=  1:
  S[9,19]  :=  1:
  S[9,26]  := -1:
  S[9,27]  :=  1:
  S[9,28]  :=  1:
  S[10,9]   := -1:
  S[10,10]  :=  1:
  S[10,78]  :=  1:
  S[10,79]  := -1:
  S[11,9]   :=  1:
  S[11,10]  := -1:
  S[11,80]  := -1:
  S[12,11]  := -1:
  S[12,12]  :=  1:
  S[12,62]  := -1:
  S[12,63]  :=  1:
  S[12,101] :=  1:
  S[12,102] := -1:
  S[13,11]  :=  1:
  S[13,12]  := -1:
  S[13,13]  := -1:
  S[14,13]  :=  1:
  S[14,14]  := -1:
  S[14,15]  :=  1:
  S[14,16]  :=  1:
  S[14,20]  := -1:
  S[14,21]  :=  1:
  S[14,23]  := -1:
  S[14,24]  :=  1:
  S[14,25]  :=  1:
  S[14,64]  :=  1:
  S[15,14]  := -1:
  S[15,15]  :=  1:
  S[15,104] :=  1:
  S[15,105] := -1:
  S[16,14]  :=  1:
  S[16,15]  := -1:
  S[16,16]  := -1:
  S[17,16]  :=  1:
  S[17,17]  := -1:
  S[17,18]  :=  1:
  S[17,19]  :=  1:
  S[17,106] := -1:
  S[18,17]  :=  1:
  S[18,18]  := -1:
  S[18,19]  := -1:
  S[19,20]  := -1:
  S[19,21]  :=  1:
  S[19,22]  :=  1:
  S[19,67]  := -1:
  S[19,68]  :=  1:
  S[19,69]  := -1:
  S[19,70]  :=  1:
  S[19,95]  :=  1:
  S[19,96]  := -1:
  S[20,20]  :=  1:
  S[20,21]  := -1:
  S[20,22]  := -1:
  S[21,23]  := -1:
  S[21,24]  :=  1:
  S[21,108] :=  1:
  S[21,109] := -1:
  S[22,23]  :=  1:
  S[22,24]  := -1:
  S[22,25]  := -1:
  S[23,25]  :=  1:
  S[23,110] := -1:
  S[24,26]  := -1:
  S[24,27]  :=  1:
  S[24,81]  :=  1:
  S[24,82]  := -1:
  S[25,26]  :=  1:
  S[25,27]  := -1:
  S[25,28]  := -1:
  S[26,28]  :=  1:
  S[26,29]  := -1:
  S[26,30]  :=  1:
  S[26,31]  := -1:
  S[26,32]  :=  1:
  S[26,33]  :=  1:
  S[27,29]  := -1:
  S[27,30]  :=  1:
  S[27,83]  :=  1:
  S[27,84]  := -1:
  S[28,29]  :=  1:
  S[28,30]  := -1:
  S[28,85]  := -1:
  S[29,31]  := -1:
  S[29,32]  :=  1:
  S[29,86]  :=  1:
  S[29,87]  := -1:
  S[30,31]  :=  1:
  S[30,32]  := -1:
  S[30,33]  := -1:
  S[31,33]  :=  1:
  S[31,34]  := -1:
  S[31,35]  :=  1:
  S[32,34]  :=  1:
  S[32,35]  := -1:
  S[32,36]  := -1:
  S[32,37]  :=  1:
  S[32,38]  := -2:
  S[32,39]  :=  2:
  S[33,36]  := -1:
  S[33,37]  :=  1:
  S[33,40]  := -1:
  S[33,41]  :=  1:
  S[33,44]  := -1:
  S[33,45]  :=  1:
  S[33,88]  :=  1:
  S[33,89]  := -1:
  S[34,36]  :=  1:
  S[34,37]  := -1:
  S[34,90]  := -1:
  S[35,38]  :=  1:
  S[35,39]  := -1:
  S[35,40]  := -1:
  S[35,41]  :=  1:
  S[35,42]  := -2:
  S[35,43]  :=  2:
  S[36,40]  :=  1:
  S[36,41]  := -1:
  S[36,91]  := -1:
  S[37,42]  :=  1:
  S[37,43]  := -1:
  S[37,44]  := -1:
  S[37,45]  :=  1:
  S[37,46]  := -1:
  S[37,47]  :=  1:
  S[38,44]  :=  1:
  S[38,45]  := -1:
  S[38,92]  := -1:
  S[39,46]  := -1:
  S[39,47]  :=  1:
  S[39,93]  :=  1:
  S[40,46]  :=  1:
  S[40,47]  := -1:
  S[40,48]  := -1:
  S[41,48]  :=  1:
  S[41,49]  := -1:
  S[41,50]  :=  1:
  S[41,51]  :=  1:
  S[41,52]  := -1:
  S[41,53]  :=  1:
  S[41,54]  :=  1:
  S[41,93]  := -1:
  S[42,49]  := -1:
  S[42,50]  :=  1:
  S[42,113] :=  1:
  S[42,114] := -1:
  S[43,49]  :=  1:
  S[43,50]  := -1:
  S[43,51]  := -1:
  S[44,51]  :=  1:
  S[44,55]  := -1:
  S[44,56]  :=  1:
  S[45,52]  := -1:
  S[45,53]  :=  1:
  S[45,97]  :=  1:
  S[45,98]  := -1:
  S[46,52]  :=  1:
  S[46,53]  := -1:
  S[46,54]  := -1:
  S[47,54]  :=  1:
  S[47,65]  := -1:
  S[47,66]  :=  1:
  S[48,55]  :=  1:
  S[48,56]  := -1:
  S[48,58]  := -1:
  S[48,59]  :=  1:
  S[48,57]  :=  1:
  S[48,115] := -1:
  S[49,58]  := -1:
  S[49,59]  :=  1:
  S[49,94]  :=  1:
  S[50,58]  :=  1:
  S[50,59]  := -1:
  S[50,57]  := -1:
  S[51,57]  :=  1:
  S[51,60]  := -1:
  S[51,61]  :=  1:
  S[51,94]  := -1:
  S[52,60]  := -1:
  S[52,61]  :=  1:
  S[53,60]  :=  1:
  S[53,61]  := -1:
  S[53,62]  := -1:
  S[53,63]  :=  1:
  S[53,64]  :=  1:
  S[53,67]  := -1:
  S[53,68]  :=  1:
  S[53,107] :=  1:
  S[54,62]  :=  1:
  S[54,63]  := -1:
  S[54,64]  := -1:
  S[55,65]  :=  1:
  S[55,66]  := -1:
  S[55,69]  := -1:
  S[55,70]  :=  1:
  S[55,99]  := -1:
  S[56,67]  :=  1:
  S[56,68]  := -1:
  S[56,107] := -1:
  S[57,69]  :=  1:
  S[57,70]  := -1:
  S[57,100] := -1:
  S[58,22]  :=  1:
  S[58,103] := -1:
  
  # System dimensions
  nx := LinearAlgebra[Dimension](S)[1];
  nk := LinearAlgebra[Dimension](S)[2];
  
  # One non-rate constant: mvol
  nc := 1:
  
  vk := Vector( nk, symbol=k ):
  vx := Vector( nx, symbol=x ):
  vc := Vector( nc, symbol=c ):
  vv := Vector( nk, symbol=v ):
  kx := Concatenate( 1, vk, vx, vc ):
  
  # Reaction fluxes
  vv(1)  := vk(1)  * vx(1) * vx(2):
  vv(2)  := vk(2)  * vx(3):
  vv(3)  := vk(3)  * vx(3):
  vv(4)  := vk(4)  * vx(4) * vx(5):
  vv(5)  := vk(5)  * vx(6):
  vv(6)  := vk(6)  * vx(4) * vx(7):
  vv(7)  := vk(7)  * vx(8):
  vv(8)  := vk(8)  * vx(8):
  vv(9)  := vk(9)  * vx(9) * vx(10):
  vv(10) := vk(10) * vx(11):
  vv(11) := vk(11) * vx(9) * vx(12):
  vv(12) := vk(12) * vx(13):
  vv(13) := vk(13) * vx(13):
  vv(14) := vk(14) * vx(14) * vx(15):
  vv(15) := vk(15) * vx(16):
  vv(16) := vk(16) * vx(16):
  vv(17) := vk(17) * vx(7) * vx(17):
  vv(18) := vk(18) * vx(18):
  vv(19) := vk(19) * vx(18):
  vv(20) := vk(20) * vx(14) * vx(19):
  vv(21) := vk(21) * vx(20):
  vv(22) := vk(22) * vx(20):
  vv(23) := vk(23) * vx(14) * vx(21):
  vv(24) := vk(24) * vx(22):
  vv(25) := vk(25) * vx(22):
  vv(26) := vk(26) * vx(9) * vx(24):
  vv(27) := vk(27) * vx(25):
  vv(28) := vk(28) * vx(25):
  vv(29) := vk(29) * vx(26) * vx(27):
  vv(30) := vk(30) * vx(28):
  vv(31) := vk(31) * vx(26) * vx(29):
  vv(32) := vk(32) * vx(30):
  vv(33) := vk(33) * vx(30):
  vv(34) := vk(34) * vx(31):
  vv(35) := vk(35) * vx(32):
  vv(36) := vk(36) * vc(1)  * vx(32) * vx(33):
  vv(37) := vk(37) * vx(34):
  vv(38) := vk(38) * vc(1)  * vx(32)^2:
  vv(39) := vk(39) * vx(35):
  vv(40) := vk(40) * vc(1)  * vx(33) * vx(35):
  vv(41) := vk(41) * vx(36):
  vv(42) := vk(42) * vc(1)  * vx(35)^2:
  vv(43) := vk(43) * vx(37):
  vv(44) := vk(44) * vc(1)  * vx(33) * vx(37):
  vv(45) := vk(45) * vx(38):
  vv(46) := vk(46) * vc(1)  * vx(39) * vx(37):
  vv(47) := vk(47) * vx(40):
  vv(48) := vk(48) * vx(40):
  vv(49) := vk(49) * vc(1)  * vx(41) * vx(42):
  vv(50) := vk(50) * vx(43):
  vv(51) := vk(51) * vx(43):
  vv(52) := vk(52) * vc(1)  * vx(41) * vx(45):
  vv(53) := vk(53) * vx(46):
  vv(54) := vk(54) * vx(46):
  vv(55) := vk(55) * vx(44):
  vv(56) := vk(56) * vx(48):
  vv(57) := vk(57) * vx(50):
  vv(58) := vk(58) * vx(48) * vx(49):
  vv(59) := vk(59) * vx(50):
  vv(60) := vk(60) * vx(51) * vx(52):
  vv(61) := vk(61) * vx(53):
  vv(62) := vk(62) * vx(12) * vx(53):
  vv(63) := vk(63) * vx(54):
  vv(64) := vk(64) * vx(54):
  vv(65) := vk(65) * vx(47):
  vv(66) := vk(66) * vx(55):
  vv(67) := vk(67) * vx(19) * vx(53):
  vv(68) := vk(68) * vx(56):
  vv(69) := vk(69) * vx(19) * vx(55):
  vv(70) := vk(70) * vx(57):
  vv(71) := vk(71):
  vv(72) := vk(72) * vx(2):
  vv(73) := vk(73):
  vv(74) := vk(74) * vx(5):
  vv(75) := vk(75) * vx(6):
  vv(76) := vk(76):
  vv(77) := vk(77) * vx(7):
  vv(78) := vk(78):
  vv(79) := vk(79) * vx(10):
  vv(80) := vk(80) * vx(11):
  vv(81) := vk(81):
  vv(82) := vk(82) * vx(24):
  vv(83) := vk(83):
  vv(84) := vk(84) * vx(27):
  vv(85) := vk(85) * vx(28):
  vv(86) := vk(86):
  vv(87) := vk(87) * vx(29):
  vv(88) := vk(88):
  vv(89) := vk(89) * vx(33):
  vv(90) := vk(90) * vx(34):
  vv(91) := vk(91) * vx(36):
  vv(92) := vk(92) * vx(38):
  vv(93) := vk(93) * vx(41):
  vv(94) := vk(94) * vx(51):
  vv(95) := vk(95):
  vv(96) := vk(96) * vx(19):
  vv(97) := vk(97):
  vv(98) := vk(98) * vx(45):
  vv(99) := vk(99) * vx(55):
  vv(100) := vk(100) * vx(57):
  vv(101) := vk(101):
  vv(102) := vk(102) * vx(12):
  vv(103) := vk(103) * vx(58):
  vv(104) := vk(104):
  vv(105) := vk(105) * vx(15):
  vv(106) := vk(106) * vx(17):
  vv(107) := vk(107) * vx(56):
  vv(108) := vk(108):
  vv(109) := vk(109) * vx(21):
  vv(110) := vk(110) * vx(23):
  vv(111) := vk(111):
  vv(112) := vk(112) * vx(1):
  vv(113) := vk(113):
  vv(114) := vk(114) * vx(42):
  vv(115) := vk(115) * vx(48):
  
  return(S,vv,vk,vx,vc):
  
end proc:
