
%
% omm2.m
%
% Derives an expression for the steady-state of the Open
% Michaelis-Menten model using a heterogeneous substitution
% strategy.
%
% Optional Arguments:
% doUnsub - if true, the py-substitution is reversed prior
%       to returning vbar. Default value is true.
% verbose - if true, each calculation in the substitution
%       and solution of the steady state equation are 
%       printed to the console. Default value is false.
% fid - a file id to which to print text when verbose=true. 
%       Returned by a previous call to fopen(). Default 
%       value is 1, the console. This argument is ignored 
%       if verbose=false.
% wid - width of the device, in characters, to which to
%       print. Default value is 80.
%
% Value:
% vbar - the steady state vector of reaction velocities 
%        satisfying S * vbar = 0. Elements in vbar are 
%        either in terms of p and q (if doUnsub is false), 
%        or k and x (if doUnsub is true).
% psi  - the composite mapping function from the set 
%        union(K,X) to union(P,Q,R[P,Q]) if doUnsub is 
%        false, or to union(K_pq,X_pq) if doUnsub is true, 
%        satisfying psi(v) = vbar.

function [vbar,psi] = omm2( varargin )

  ip = inputParser;
  addParamValue(ip,'doUnsub',true,@islogical);
  addParamValue(ip,'verbose',false,@islogical);
  addParamValue(ip,'fid',1,@isnumeric);
  addParamValue(ip,'wid',80,@isnumeric);

  parse(ip,varargin{:});
  doUnsub = ip.Results.doUnsub;
  verbose = ip.Results.verbose;
  fid = ip.Results.fid;
  wid = ip.Results.wid;

  % Load the model description
  [S,v,k,x,xdot] = loadOMM();
  
  if( verbose )
    printvar(sprintf(strcat('\nThere are %d reactions',...
      ' and %d species in the OMM model.\n'),...
      length(k), length(x)), 'fid', fid, 'wid', wid );
    printvar(sprintf(strcat('The stoichiometric matrix is S =\n')),...
      'fid', fid, 'wid', wid);
    printvar( S, 'vsym', 'S', 'fid', fid, 'wid', wid );
    printvar(sprintf(strcat('\nThe vector of reaction',...
      ' velocities is v, where\n')),'wid',wid,'fid',fid);
    printvar( v, 'vsym', 'v', 'fid', fid, 'wid', wid );
    printvar(sprintf(strcat('\nThe vector of mass balance',...
      ' equations is xdot = S*v, where\n')),'wid',wid,'fid',fid);
    printvar( xdot, 'vsym', 'xdot', 'fid', fid, 'wid', wid );
  end

  % Declare vectors of in/dependent variables
  p = sym( 'p', [5,1] ); y = sym( 'y', [4,1] );
  p = sym( p, 'positive' ); p = sym( p, 'real' );
  y = sym( y, 'positive' ); y = sym( y, 'real' );
  
  % Substitution strategy
  kx = [ k' x' ];
  py = [ p(2:4)',y(4),p(5),y(1),p(1),y(2:3)' ];
  
  % Execute pysub
  [vbar,ybar,psi_py,q] = pysub( S, v, kx, py, y );

  % For consistency with Maple, want ybar(4)=q(1)
  streq = strcat('a*',char(ybar(4)),'=',char(q(1)));
  a = solve( streq, 'a' );
  Myq = a*q;

  % Re-execute pysub  
  [vbar,ybar,psi_py,q] = pysub( S, v, kx, py, y, 'q', Myq,...
    'verbose', verbose, 'fid', fid, 'wid', wid );

  % Reverse the substitution
  if( doUnsub )
    [vbar,psi_ss] = unsub( v,p,q,y,kx,py,ybar,psi_py,...
      'verbose', verbose, 'fid', fid, 'wid', wid );
    psi = psi_ss;
  else
    psi = psi_py;
  end
  
  % Verify steady state
  if( verbose )
    printvar(sprintf(strcat('\nVerify steady state.',...
      ' xdot = S * vbar, where\n')), 'fid', fid, 'wid', wid); 
    printvar( simplify(S*vbar), 'vtype', 'vec', 'vsym', 'xdot',...
      'fid', fid, 'wid', wid );
  end
