
# 
# omm2.mpl
# 
# Maple implementation of a heterogeneous py-substitution 
# strategy for the OMM model, where a subset of species 
# abundances and reaction rate constants are mapped to
# each set of indpendent and dpeparameters.
# 
# Optional Arguments:
# doUnsub - if true, the py-substitution is reversed
#      prior to returning vbar. Default value is true.
# verbose - if true, each calculation in the substitution
#      and solution of the steady state equation are
#      printed to the console. Default value is false.
# fid - the active file id to which to print var.
# wid - width of the device, in characters, to which to
#       print. Default value is 80.
#
# Value:
# vbar - the reaction velocity vector psi(v) where
#        S * vbar = 0
# psi  - the steady state mapping function psi(v)=vbar.
#        Depending on the value of doUnsub, this will
#        either be the forward mapping function psi_py
#        or the composite function psi_ss.

omm2 := proc()

  local S, vv, vk, vx, vc, vp, vy, vq, Myvq, a:
  local ns, nk, nx, nc, np, ny:
  local kx, py, PI:
  local kxbar, ybar, vbar, psi:
  local Options, Defaults:
  local pverb, punsub, pfid, pwid:

  read "printvar.mpl":
  read "pysub.mpl":

  # Parse out optional parameters
  Options := table():
  Defaults := table([
    OptionParms=table([
      'doUnsub'=[true,boolean],
      'verbose'=[false,boolean],
      'wid'=[80,integer],
      'fid'=[-1,integer]
    ]),PositionalParms=table()]):

  ProcessOptions( 0, [args], Defaults, Options );
  pverb := Options[verbose]:
  punsub := Options[doUnsub]:
  pwid := Options[wid]:
  pfid := Options[fid]:
  

  #
  # Since the OMM model is only used once in Maple,
  # we won't bother with a loadOMM() funciton, but 
  # rather just define it here.
  #

  S := <
    -1, 1, 1, 0, 0;
    -1, 1, 0, 1, 0;
     1,-1,-1, 0, 0;
     0, 0, 1, 0,-1
  >:

  # System dimensions
  nx := LinearAlgebra[Dimension](S)[1]:
  nk := LinearAlgebra[Dimension](S)[2]:
  nc := 0:
  ns := nx + nk + nc:

  vk := Vector( nk, symbol=k ):
  vx := Vector( nx, symbol=x ):
  vc := Vector( nc, symbol=c ):
  vv := Vector( nk, symbol=v ):
  kx := ArrayTools[Concatenate]( 1, vk, vx, vc ):

  # Reaction fluxes
  vv(1) := vk(1) * vx(1) * vx(2):
  vv(2) := vk(2) * vx(3):
  vv(3) := vk(3) * vx(3):
  vv(4) := vk(4):
  vv(5) := vk(5) * vx(4):


  #
  # Pysub strategy
  #

  np := 5:
  ny := ns - np:

  vp := Vector( np, symbol=p ):
  vy := Vector( ny, symbol=y ):
  py := ArrayTools[Concatenate]( 1, vp, vy ):

  PI := Matrix( ns ):
  PI(1,7) := 1:
  PI(2,1) := 1:
  PI(3,2) := 1:
  PI(4,3) := 1:
  PI(5,5) := 1:
  PI(6,6) := 1:
  PI(7,8) := 1:
  PI(8,9) := 1:
  PI(9,4) := 1:

  # Print out model and substitution strategy
  if pverb then
    printvar(sprintf("\nThere are %d reactions and %d"
      " species in the OMM model.\n",nk,nx),fid=pfid,wid=pwid):
    printvar("The stoichiometric matrix is S =\n", 
      fid=pfid, wid=pwid ):
    printvar( S, vsym="S", fid=pfid, wid=pwid ):
    printvar(sprintf("\nThe vector of reaction"
      " velocities is v, where\n"),fid=pfid,wid=pwid):
    printvar( vv, vsym="v", fid=pfid, wid=pwid ):
    printvar(sprintf("\nThe vector of mass balance"
      " equations is xdot = S*v, where \n"),fid=pfid,wid=pwid):
    printvar( LinearAlgebra[Multiply](S,vv),
      vsym="xdot", fid=pfid, wid=pwid ):
  end if:
  
  # Perform the substitution
  vbar,ybar,psi,vq := Pysub( S, vv, kx, py, vy, PI ):

  # For consistency with Matlab, want ybar[4]=q[1]
  a := solve( a*ybar[4]=q[1], a ):
  Myvq := a*vq:
  
  # Perform the substitution again
  vbar,ybar,psi,vq := Pysub( S, vv, kx, py, vy, PI,
    'vq'=Myvq, verbose=pverb, wid=pwid, fid=pfid ):

  # Unsub, if requested
  if punsub then
    vbar,psi := Unsub( vv, vp, vq, vy, kx, py, ybar, 
      PI, verbose=pverb, wid=pwid, fid=pfid ):
  end if:

  # Verify steady state
  if pverb then
    printvar(sprintf("\nVerify steady state. xdot ="
      " S * vbar, where\n"),fid=pfid,wid=pwid ): 
    printvar( simplify(LinearAlgebra[Multiply](S,vbar)), 
      vtype=VECTOR, vsym="xdot", fid=pfid, wid=pwid ):
  end if:
  
  return(vbar,psi):

end proc:
