
#
# pysub.mpl
#
# Solves,
#
#     S * vv = 0
#
# where vv is a vector of mass action reaction velocities described
# in terms of elements in the vectors vk, vx, and optionally, vc. 
# Returns a mapping function psi_py and vector vbar where
#
#     vbar = psi_py( vv )
#
# and
#
#     S * vbar = 0.
# 
# The vector ybar is returned to facilitate alternative steady
# state expressions. Note that
#
#     ybar = N * vq
#
# where N is the matrix whose column vectors form a basis for the
# coefficient matrix C = S * P, where P is the jacobian of the
# linearized velocity vector vv = P * vy.
#
# Arguments:
# S  - a stoichiometric matrix. Element s_{i,j} gives the 
#      stoichiometry of species i in reaction j.
# vv - a symbolic vector of biochemical reaction velocities
# kx - a concatenated vector of symbolic reaction rate constants,
#      species concentrations, and additional constants appearing
#      in vv.
# py - a concatenated vector of symbolic independent parameters
#      and dependent variables
# vy - a symbolic vector of dependent variables
# PI - a permutation matrix kx = PI^T * py. Defines
#      a map psi_p : kx(i) |--> py(i) such that v is linear in y.
#
# Optional Arguments:
# vq  - a vector of null space basis coefficients
# verbose - if true, each calculation in the substitution and
#       solution of the steady state equation are printed to the 
#       console. Default value is false.
# fid - a file id to which to print text when verbose=true. Returned
#       by a previous call to fopen(). Default value is 'terminal'. 
#       This argument is ignored if verbose is false.
# wid - width of the device, in characters, to which to print.
#       Default value is 80.
# doCrref - if true, the reduced row echelon form of C will be
#       calculated. This matrix is instructional in that it
#       illustrates how the matrix N is derived from C, but it
#       is not required for the calculation of N and can, for
#       large models, consume a lot of resources. Default value
#       is true. Note that this step is automatically skipped
#       if verbose=false.
# 
# Value:
# vbar - a steady state vector of reaction velocities, S * vbar = 0.
# ybar - a linear combination of null space basis vectors. See above.
# psi_py - a two-column array where psi_py(i,2) is the steady state
#        expression for model element psi_py(i,1).
# vq   - the vector of null space basis vector coefficients,
#        ybar = N * q
#

Pysub := proc(S,vv,kx,py,vy,PI)

  local pq, pqy, nq, ns:
  local P, C, N:
  local Clist, Nlist:
  local pysub, ybar, vbar:
  local novq, vqloc, vqdef:
  local i:
  local dgts, sfmt, psi_p, psi_pv, psi_py:
  local Options, Defaults:
  local pverb, pfid, pwid, pdoCrref:
  
  read "printvar.mpl":
 
  # Parse out optional parameters
  Options := table():
  Defaults := table([
    OptionParms=table([
      'verbose'=[false,boolean],
      'wid'=[80,integer],
      'fid'=[-1,integer],
      'vq'=[Vector(0,symbol='q'),'Vector'],
      'doCrref'=[true,boolean]
    ]),PositionalParms=table()]):
    
  ProcessOptions( 1, [args], Defaults, Options );
  pverb := Options['verbose']:
  pfid := Options['fid']:
  pwid := Options['wid']:
  pdoCrref := Options['doCrref']:
  vqloc := Options['vq']:
 

  # Pysub vectors
  ns := LinearAlgebra[Dimension](kx):
  
  # String width when outputting text
  dgts := max( 
    LinearAlgebra[Dimension](vv), LinearAlgebra[Dimension](vy),
    LinearAlgebra[Dimension](kx)-LinearAlgebra[Dimension](vv), 
    LinearAlgebra[Dimension](py)-LinearAlgebra[Dimension](vy) ):
  sfmt := sprintf("%%-%da",floor(log10(dgts))+2);

  # Perform the substitution
  psi_p := ArrayTools[Concatenate]( 2, kx, 
    LinearAlgebra[Multiply](LinearAlgebra[Transpose](PI),py) ):
  pysub := Equate( psi_p[..,1], psi_p[..,2] ):
  
  if pverb then
    printvar( "\nLet the map psi_p be given by\n", fid=pfid,wid=pwid ):
    printvar( psi_p, vtype=MAP, wid=pwid, fid=pfid ):
    printvar( "\nThis results in a linear velocity vector psi_p[v], where\n",
      fid=pfid, wid=pwid ):
    psi_pv := subs(pysub,vv):
    printvar( psi_pv, vsym="psi_p[v[%s]]", wid=pwid, fid=pfid ):
  end if:

  # Create the P-matrix
  P := VectorCalculus[Jacobian]( subs(pysub,vv), 
    convert(vy,list) ):
    
  if pverb then
    printvar( cat("\nWe can express psi_p[v] as the product P*y,"
      " where y is the vector [y1,...,",
      sprintf("y%d",LinearAlgebra[Dimension](vy)),
      "]^T and P =\n"), fid=pfid, wid=pwid ):
    printvar( P, vtype=MATRIX, vsym="P", fid=pfid, wid=pwid ):
  end if:

  # Create the coefficient matrix
  C := LinearAlgebra[Multiply](S,P):
  
  if pverb then
    printvar("\nFrom this we calculate the coefficient matrix, C = S*P =\n",
      fid=pfid, wid=pwid ):
    printvar( C, vtype=MATRIX, vsym="C", wid=pwid, fid=pfid ):
    
    if pdoCrref then
      printvar("\nC is row equivalent to the reduced matrix Crref =\n",
        fid=pfid, wid=pwid ):
      printvar( LinearAlgebra[ReducedRowEchelonForm](C),
        vtype=MATRIX, vsym="Crref", wid=pwid, fid=pfid ):
    end if:
  end if:

  # Calculate its kernel
  Clist := convert( C, list, nested=true ):
  Nlist := find_nullspace_vecform( Clist ):
  N := LinearAlgebra[Transpose]( convert( Nlist, Matrix ) ):
  
  if pverb then
    printvar("\nThe null space of C is spanned by the columns of N =\n",
      fid=pfid, wid=pwid ):
    printvar( N, vtype=MATRIX, vsym="N", wid=pwid, fid=pfid ):
  end if:

  # Calculate y=f(p)
  nq := LinearAlgebra[Dimension](N)[2]:
  vqdef := Vector( nq, symbol=q ):
  if LinearAlgebra[Dimension](vqloc)=0 then
    vqloc := vqdef:
  end if:
  ybar := LinearAlgebra[Multiply](N,vqloc):
  
  if pverb then
    printvar( "\nLet ybar = N*q, where q is given by,"
      " where q is given by\n", wid=pwid, fid=pfid ):
    printvar( vqloc, vtype=VECTOR, vsym="q", fid=pfid, wid=pwid ):
    printvar("\nThis gives\n", fid=pfid, wid=pwid ):
    printvar( ybar, vtype=VECTOR, vsym="ybar", wid=pwid, fid=pfid ):
  end if:


  # Form the py-vector satisfying steady state
  pqy := subs( Equate(vy,ybar), py ):
  
  # Derive the steady state reaction velocity vector
  psi_py := ArrayTools[Concatenate](2,kx,LinearAlgebra[Multiply](
    LinearAlgebra[Transpose](PI),pqy)):
  vbar := subs( Equate(psi_py[..,1],psi_py[..,2]), vv ):
  
  if pverb then
    printvar( "\nFrom ybar we construct the composite forward map psi_py :\n",
      fid=pfid, wid=pwid ):
    printvar( psi_py, vtype=MAP, wid=pwid, fid=pfid ):
    printvar("\nThe steady state reaction velocity vector vbar"
      " is given by psi_py[v], where\n", fid=pfid, wid=pwid ):
    printvar( vbar, vtype=VECTOR, vsym="vbar", wid=pwid, fid=pfid ):
  end if:

  # Always return the default vq vector
  vqloc := vqdef:
  return (vbar,ybar,psi_py,vqloc):

end proc:



#
# unsub.mpl
#
# Generates and returns the composite steady state map
# psi_ss = ( psi_{qp}^{-1} \dot psi_{py} ) where
#
#     vbar = psi_ss( vv )
#
# and
#
#     S * vbar = 0.
#
# Call unsub() after a call to pysub() to map steady
# state expressions back to elements in kx.
#
# Arguments:
# vv - the original vector of reaction velocities, with
#      elements defined in terms of k and x
# vp - a vector of independent parameters
# vq - the vector of basis vector coefficients returned
#      by pysub(). Note this should NOT be a custom vector
#      of coefficients passed in to pysub().
# vy - a vector of dependent parameters
# kx - the concatentated vector of rate constants and
#      species abundances
# py - the vector py used in a preceding call to pysub()
# ybar - the vector of dependent parameters that results
#      from solving the steady state equation, as returned
#      by pysub()
# PI - the permutation matrix kx = PI^T * py used in a
#      preceding call to pysub().
#
# Optional Arguments:
# verbose - if true, the mapping functions from sets P, Q, and Y 
#       back to K and X are printed out, along with the unsubstituted 
#       stead state velocity vector. Default value is false.
# fid - a file id to which to print text when verbose=true. Returned
#       by a previous call to fopen(). Default value is 'terminal'. 
#       This argument is ignored if verbose is false.
# wid - width of the device, in characters, to which to print.
#       Default value is 80.
#
# Value:
# vbar - the reaction velocity vector vbar=psi_ss(v),
#        where S * vbar = 0
# psi_ss - the composite forward and backward map that 
#        linearizes the vector v via the forward map
#        psi_py, solves the steady state equation, then
#        reverses the substitution.
# 

Unsub := proc( vv, vp, vq, vy, kx, py, ybar, PI )

  local PHI, qp, pybar:
  local unsub, vbar:
  local Options, Defaults:
  local pverb, pfid, pwid:
  local vqlist, vqvector, vqindets, wrongvq, dq, i, j:
  local psi_q, psi_qp, psi_ss:

  # Parse out optional parameters
  Options := table():
  Defaults := table([
    OptionParms=table([
      'verbose'=[false,boolean],
      'wid'=[80,integer],
      'fid'=[-1,integer]
    ]),PositionalParms=table()]):

  ProcessOptions( 8, [args], Defaults, Options );
  pverb := Options[verbose]:
  pfid := Options[fid]:
  pwid := Options[wid]:


  # Check vq
  vqvector := sort(convert(vq,list)):
  vqindets := sort(convert(indets(vq),list)):
  dq := nops(vqindets):
  wrongvq := false:
  if dq <> nops(vqvector) then
    wrongvq := true:
  else
    for i from 1 to dq do
      if vqvector[i] <> vqindets[i] then
        wrongvq := true:
      end if:
    end do:
  end if:
  if wrongvq then
    error "Argument vq must be the vector where vq[i]=q[i], "
      "i = 1..dq, as returned by pysub().\n":
  end if:

  # Derive the maping function psi_q
  psi_q := Matrix(dq,2): j := 1:
  vqlist := convert(vq,list):
  for i from 1 to LinearAlgebra[Dimension](ybar) do
    if member(ybar[i],vqlist) then
      psi_q[j] := Vector[row]([ybar[i],vy[i]]):
      j := j + 1:
    end if:
  end do:
  if pverb then
    printvar("\nThe mapping function psi_q^-1 is given by\n",
      fid=pfid, wid=pwid):
    printvar( psi_q, vtype=MAP, fid=pfid, wid=pwid ):
  end if:

  # Generate PHI matrix
  pybar := ArrayTools[Concatenate](1,vp,ybar):
  qp := ArrayTools[Concatenate](1,vp,vq):
  PHI := calcPHI(pybar,qp,PI):

  # Composite inverse map
  unsub := Equate( qp, LinearAlgebra[Multiply](PHI,kx) ):
  psi_qp := ArrayTools[Concatenate](2, qp, subs(unsub,qp)):
  if pverb then
    printvar("\nThe composite inverse map psi_qp^-1:\n",
      fid=pfid, wid=pwid ):
    printvar( psi_qp, vtype=MAP, fid=pfid, wid=pwid ):
  end if:
  
  # Final steady state map
  psi_ss := ArrayTools[Concatenate]( 2, kx, subs(unsub,
    LinearAlgebra[Multiply](LinearAlgebra[Transpose](PI),pybar)) ):
  if pverb then
    printvar("\nThe complete steady state map psi_ss is therefore\n",
      fid=pfid, wid=pwid ):
    printvar( psi_ss, vtype=MAP, fid=pfid, wid=pwid ):
  end if:
  
  # Steady state velocity vector
  vbar := subs(Equate(psi_ss[..,1],psi_ss[..,2]),vv):
  if pverb then
    printvar("\nThe unsubstituted steady state reaction velocity"
      "vector vbar = psi_ss[v] is given by\n", wid=pwid, fid=pfid ):
    printvar( vbar, vtype=VECTOR, vsym="vbar", wid=pwid, fid=pfid ):
  end if:

  return( vbar, psi_ss ):

end proc:



#
# Generates a matrix PHI that maps independent variables
# in the vector pq to the original system elements in kx.
#
# Arguments:
# pybar - the concatenated vector [ vp, ybar ]^T
# qp - the concatenated vector [ vp vq ]^T
# PI - the permuation matrix specifying the substitution
# 

calcPHI := proc(pybar,qp,PI)

  local PHI:
  local i,j,imax:
  local pq, ns:

  # Derive pq vector
  pq := indets( pybar ) intersect indets(qp):

  # The injective g:pq -> kx
  imax := LinearAlgebra[Dimension](pybar):
  PHI := []:
  for i in seq(1..imax) do
    if member(pybar[i],pq) then
      PHI := ArrayTools[Concatenate](1,PHI,PI[i]):
    fi;
  od;
  
  return PHI:

end:



#  sol1_x := find_nullspace_eqform(m1);
#    --- The nullspace consists of certain vectors
#        [ x[1], x[2], ..., x[c] ]'
#        Certain of the x[i]'s are free variables.
#        The rest are dependent variables.
#        sol_x consists of equations
#          v = v    when v is a free variable
#          v = expression in terms of free variables  when v is dependent
#        so substituting sol_x into [x[1],...,x[c]]' gives the
#        general solution, and substituting values for the free variables
#        gives particular solutions.
#
#  simplify_solution(sol1_x):
#        This puts the equations into the form
#          x[i1] = (factored expression)*x[i2] + (factored expr)*x[i3] + ...
#
#
#  sol1_v := find_nullspace_vecform(m1);
#   --- Gives ONE of the many possible bases of the solution space.
#       Returns [ [b1[1],...,b1[c]], [b2[1],...,b2[c]], ...]
#       where [bK[1],...,bK[c]]' is a column vector giving Kth basis vector



# Find nullspace by converting to Maple's equation solver
#
# find_nullspace_eqform(m):
#   Nullspace is vectors [ x[1] x[2] ... ]'
#   and this returns a set of equations in the x[i]'s
#
#   Equations
#     x[i] = x[i]
#   mean it's a free variable that can be set arbitrarily.
#
#   Equations
#     x[i] = something else
#   mean it's a dependent variable.
#
# find_nullspace_vecform(m):
#   This picks out a particular basis of the nullspace
#   by taking each free variable one at a time, setting it to 1,
#   and setting the other variables to 0.

find_nullspace_eqform := proc(m)
    local nrows, ncols, eqs, x_list, sols;

    nrows := nops(m);
    ncols := nops(m[1]);
    eqs := {'sum('m[i][j]*x[j]', 'j'=1..ncols)'
             $'i'=1..nrows};

    x_list := { 'x[j]' $ 'j'=1..ncols };

    sols := [solve(eqs, x_list)];

    # Solve returns an expression sequence of sets.
    # sols[k] is ambiguous because if it's one set, it refers
    # to the kth element of it, but if it's 0 or >=2 sets, it
    # refers to the kth set.  So I made it into a list with the
    # [solve(...)]
    # However, for a linear system, there should be exactly 0 sets
    # or 1 set in this list.

    if (nops(sols)=0) then
        print("No solutions");
        return NULL;
    fi;

    if (nops(sols)>1) then
        print("Something's wrong: A linear system of equations should have one solution set");
    fi;

    # only use the first set of solutions.
    sols := sols[1];
    return sols;
end:

simplify_solution := proc(eqs)
    local x_list;
    x_list := select(has,indets(m,indexed),x);
    collect(eqs,x_list,factor);
end:


find_nullspace_vecform := proc(m)
    local sols,
          free_vars, dep_eqs,
          basis,
          ncols, x_vec,
          v, sol_v,
          free_subs, dep_subs;

    sols := find_nullspace_eqform(m);

    # The equations in the solution have the form
    #   v = v
    # when it's a free variable, or
    #   v = some expression not involving v
    # when it's a dependent variable.
    #
    # So, get a list of free variables by picking out all the
    # tautological equations (note "v = v" evaluates to true, others don't)

    free_vars := map(p -> if evalb(p) then lhs(p) else NULL fi, sols);

    # and get a list of the dependent equations
    dep_eqs := map(p -> if (not evalb(p)) then p else NULL fi, sols);

    basis := NULL;

    ncols := nops(m[1]);
    x_vec := [ 'x[i]' $ 'i'=1..ncols ];

    for v in free_vars do
        free_subs := map((p,v0) -> if (evalb(p=v0)) then p=1 else p=0 fi,
                         free_vars, v);
        dep_subs := map(factor,subs(free_subs, dep_eqs));
        sol_v := subs(free_subs, dep_subs, x_vec);
        basis := basis, sol_v;
    od;

    return [basis];
end:
